package net.knarcraft.minecraftserverlauncher.server;

/**
 * A more advanced servertype for particularly tricky jar downloads.
 *
 * @author      Kristian Knarvik <kristian.knarvik@knett.no>
 * @version     1.0.0
 * @since       1.0.0
 */
public class AdvancedServerType extends ServerType {
    private final String versionURL;
    private final String downloadURLPart;
    private final String srcStart;
    private final String srcEnd;

    /**
     * Instantiates a new Advanced server type
     * @param name <p>The name of the server type</p>
     * @param versions <p>A list of one or more server versions for the type</p>
     * @param versionURL <p>The URL for checking last version for server type</p>
     * @param srcStart <p>The string in the version file marking the start of the newest version entry</p>
     * @param srcEnd <p>The string in the version file marking the end of the newest version entry</p>
     * @param downloadURL <p>The URL used for downloading .jar files</p>
     * @param downloadURLPart <p>An extra part for the download URL</p>
     */
    AdvancedServerType(String name, String[] versions, String versionURL, String srcStart, String srcEnd, String downloadURL, String downloadURLPart) {
        super(name, versions, downloadURL);
        this.srcStart = srcStart;
        this.srcEnd = srcEnd;
        this.versionURL = versionURL;
        this.downloadURLPart = downloadURLPart;
    }

    /**
     * Gets the URL used for downloading latest version information
     * @return <p>The latest version URL</p>
     */
    public String getVersionURL() {
        return this.versionURL;
    }

    /**
     * Gets an additional part of the download URL
     * @return <p>Additional download URL part</p>
     */
    public String getDownloadURLPart() {
        return this.downloadURLPart;
    }

    /**
     * Gets the string marking the start of the latest server version in the version document
     * @return <p>A string marking the start of the latest version</p>
     */
    public String getSrcStart() {
        return this.srcStart;
    }

    /**
     * Gets the string marking the end of the latest server version in the version document
     * @return <p>A string marking the end of the latest version</p>
     */
    public String getSrcEnd() {
        return this.srcEnd;
    }
}
